###############################################################################
# zapplet - A tray applet for interfacing with Zenoss
# Copyright (C) 2008 Nathaniel McCallum <nathaniel@natemccallum.com>
# 
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2.1 of the License, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT 
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
# 
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
###############################################################################

class OSFunctions(object):

	_osf = None

	DEFAULTS = {
		"hostname": "",
		"port":     8080,
		"username": "admin",
		"password": "zenoss",
		"ssl":      False,
		"interval": 5,
	}

	@classmethod
	def factory(cls):
		if cls._osf:
			return cls._osf

		for subcls in cls.__subclasses__():
			try:
				cls._osf = subcls()
				return cls._osf
			except (NotImplementedError, ImportError):
				pass
		raise NotImplementedError()

	def listServers(self):
		raise NotImplementedError()

	def addServer(self, server):
		raise NotImplementedError()

	def delServer(self, server):
		raise NotImplementedError()

	def get(self, server, key):
		raise NotImplementedError()

	def set(self, server, key, value):
		raise NotImplementedError()

	def play(self, sound_file):
		raise NotImplementedError()

class LinuxOSFunctions(OSFunctions):
	"""A class providing GNOME/Linux related functionality"""

	GCONF_PREFIX = "/apps/zapplet"

	def _get_path(self, server, key=None):
		path  = self.__class__.GCONF_PREFIX
		path += "/" + server + (key and ("/%s" % key) or "")
		return path

	def __init__(self):
		import gconf
		self._client = gconf.client_get_default()

	def listServers(self):
		return [x.split("/")[-1] for x in self._client.all_dirs(self.__class__.GCONF_PREFIX)]

	def addServer(self, server):
		if server in self.listServers(): raise KeyError()
		path = self._get_path(server, "hostname")
		print path
		self._client.set_value(path, "")

	def delServer(self, server):
		import gconf
		self._client.recursive_unset(self._get_path(server), gconf.UNSET_INCLUDING_SCHEMA_NAMES)

	def get(self, server, key):
		if not server in self.listServers():         raise KeyError()
		if not self.__class__.DEFAULTS.has_key(key): raise KeyError()

		try:
			return self._client.get_value(self._get_path(server, key))
		except ValueError:
			if self.__class__.DEFAULTS[key] != None:
				return self.__class__.DEFAULTS[key]
			else:
				raise
		
	def set(self, server, key, value):
		if not server in self.listServers():                  raise KeyError()
		if not self.__class__.DEFAULTS.has_key(key):          raise KeyError()
		if type(value) != type(self.__class__.DEFAULTS[key]): raise ValueError()

		self._client.set_value(self._get_path(server, key), value)

	def play(self, sound_file):
		raise NotImplementedError()

class MacOSXOSFunctions(OSFunctions):
	"""A class providing Mac OS X related functionality"""

	def __init__(self):
		raise NotImplementedError

class WindowsOSFunctions(OSFunctions):
	"""A class providing Windows related functionality"""

	def __init__(self):
		raise NotImplementedError
